@@ -964,6 +964,9 @@ arv_uvcp_command_get_type (void)
 			{ ARV_UVCP_COMMAND_PENDING_ACK,
 			  "ARV_UVCP_COMMAND_PENDING_ACK",
 			  "pending-ack" },
+			{ ARV_UVCP_COMMAND_EVENT_CMD,
+			  "ARV_UVCP_COMMAND_EVENT_CMD",
+			  "event-cmd" },
 			{ 0, NULL, NULL }
 		};
 		the_type = g_enum_register_static (
