@@ -4,6 +4,10 @@
 #ifndef ARV_ENUM_TYPES_H
 #define ARV_ENUM_TYPES_H
 
+#if !defined (ARV_H_INSIDE) && !defined (ARAVIS_COMPILATION)
+#error "Only <arv.h> can be included directly."
+#endif
+
 #include <glib-object.h>
 
 G_BEGIN_DECLS
