@@ -23,6 +23,10 @@
 #ifndef ARV_GC_INTEGER_NODE_H
 #define ARV_GC_INTEGER_NODE_H
 
+#if !defined (ARV_H_INSIDE) && !defined (ARAVIS_COMPILATION)
+#error "Only <arv.h> can be included directly."
+#endif
+
 #include <arvtypes.h>
 #include <arvgcfeaturenode.h>
 #include <arvgcpropertynode.h>
