@@ -703,7 +703,7 @@ _load_genicam (ArvGvDevice *gv_device, guint32 address, size_t  *size)
 
 					if (arv_debug_check (&arv_debug_category_misc, ARV_DEBUG_LEVEL_LOG)) {
 						GString *string = g_string_new ("");
-						
+
 						g_string_append_printf (string,
 									"[GvDevice::load_genicam] Raw data size = 0x%x\n", file_size);
 						arv_g_string_append_hex_dump (string, genicam, file_size);
@@ -847,6 +847,14 @@ arv_gv_device_load_genicam (ArvGvDevice *gv_device)
 					      "<AccessMode>RO</AccessMode>"
 					      "<pPort>Device</pPort>"
 					      "</StringReg>");
+		arv_gc_set_default_node_data (gv_device->priv->genicam, "NumberOfStreamChannels",
+					      "<IntReg Name=\"NumberOfStreamChannels\">"
+					      "<Address>0x904</Address>"
+					      "<Length>4</Length>"
+					      "<AccessMode>RO</AccessMode>"
+					      "<Endianess>BigEndian</Endianess>"
+					      "<pPort>Device</pPort>"
+					      "</IntReg>");
 		arv_gc_set_default_node_data (gv_device->priv->genicam, "GevSCPHostPort",
 					      "<Integer Name=\"GevSCPHostPort\">"
 					      "<Visibility>Expert</Visibility>"
@@ -964,6 +972,18 @@ arv_gv_device_load_genicam (ArvGvDevice *gv_device)
 					      "<pVariable Name=\"SEL\">GevStreamChannelSelector</pVariable>"
 					      "<Formula>SEL * 0x40</Formula>"
 					      "</IntSwissKnife>");
+		arv_gc_set_default_node_data (gv_device->priv->genicam, "GevStreamChannelSelector",
+					      "<Integer Name=\"GevStreamChannelSelector\">"
+					      "<Value>0</Value>"
+					      "<Min>0</Min>"
+					      "<pMax>GevStreamChannelSelectorMax</pMax>"
+					      "<Inc>1</Inc>"
+					      "</Integer>");
+		arv_gc_set_default_node_data (gv_device->priv->genicam, "GevStreamChannelSelectorMax",
+					      "<IntSwissKnife Name=\"GevStreamChannelSelectorMax\">"
+					      "<pVariable Name=\"N_STREAM_CHANNELS\">NumberOfStreamChannels</pVariable>"
+					      "<Formula>N_STREAM_CHANNELS - 1</Formula>"
+					      "</IntSwissKnife>");
 		arv_gc_set_default_node_data (gv_device->priv->genicam, "TLParamsLocked",
 					      "<Integer Name=\"TLParamsLocked\">"
 					      "<Visibility>Invisible</Visibility>"
@@ -986,7 +1006,7 @@ arv_gv_device_create_stream (ArvDevice *device, ArvStreamCallback callback, void
 	GInetAddress *interface_address;
 	GInetAddress *device_address;
 
-	arv_device_read_register (device, ARV_GVBS_N_STREAM_CHANNELS_OFFSET, &n_stream_channels, NULL);
+	n_stream_channels = arv_device_get_integer_feature_value (device, "NumberOfStreamChannels");
 	arv_debug_device ("[GvDevice::create_stream] Number of stream channels = %d", n_stream_channels);
 
 	if (n_stream_channels < 1)
@@ -1005,7 +1025,7 @@ arv_gv_device_create_stream (ArvDevice *device, ArvStreamCallback callback, void
 		return NULL;
 
 	if (!gv_device->priv->is_packet_resend_supported)
-		g_object_set (stream, "packet-resend", ARV_GV_STREAM_PACKET_RESEND_NEVER, NULL); 
+		g_object_set (stream, "packet-resend", ARV_GV_STREAM_PACKET_RESEND_NEVER, NULL);
 
 	return stream;
 }
