@@ -23,6 +23,10 @@
 #ifndef ARV_INTERFACE_H
 #define ARV_INTERFACE_H
 
+#if !defined (ARV_H_INSIDE) && !defined (ARAVIS_COMPILATION)
+#error "Only <arv.h> can be included directly."
+#endif
+
 #include <arvtypes.h>
 #include <arvdevice.h>
 
