@@ -23,6 +23,10 @@
 #ifndef ARV_TYPES_H
 #define ARV_TYPES_H
 
+#if !defined (ARV_H_INSIDE) && !defined (ARAVIS_COMPILATION)
+#error "Only <arv.h> can be included directly."
+#endif
+
 #include <arvenums.h>
 #include <arvfeatures.h>
 
@@ -46,7 +50,6 @@ typedef struct _ArvGc ArvGc;
 typedef	struct _ArvGcNode			ArvGcNode;
 typedef struct _ArvGcPropertyNode		ArvGcPropertyNode;
 typedef struct _ArvGcIndexNode			ArvGcIndexNode;
-typedef struct _ArvGcVariableNode		ArvGcVariableNode;
 typedef struct _ArvGcInvalidatorNode		ArvGcInvalidatorNode;
 typedef struct _ArvGcFeatureNode 		ArvGcFeatureNode;
 typedef struct _ArvGcRegisterDescriptionNode 	ArvGcRegisterDescriptionNode;
