@@ -1,6 +1,8 @@
 #include <glib.h>
 #include <arv.h>
 #include <string.h>
+
+#define ARAVIS_COMPILATION
 #include "../src/arvbufferprivate.h"
 
 typedef struct {
@@ -311,6 +313,12 @@ swiss_knife_test (void)
 	value = arv_gc_integer_get_value (ARV_GC_INTEGER (node), NULL);
 	g_assert_cmpint (value, ==, 4);
 
+	node = arv_gc_get_node (genicam, "IntSwissKnifeTestSubAndConstant");
+	g_assert (ARV_IS_GC_SWISS_KNIFE (node));
+
+	value = arv_gc_integer_get_value (ARV_GC_INTEGER (node), NULL);
+	g_assert_cmpint (value, ==, 140);
+
 	g_object_unref (device);
 }
 
